/*
  DL1414T.cpp - Library for accessing the Siemens DL1414T display.
  Author: Sven Maerivoet
  Last modified: 05/08/2012
  Target: Arduino Uno rev3
*/

#include "Arduino.h"
#include "DL1414T.h"

// constructor
DL1414T::DL1414T()
{
  pinMode(kD0DataInput,OUTPUT);
  pinMode(kD1DataInput,OUTPUT);
  pinMode(kD2DataInput,OUTPUT);
  pinMode(kD3DataInput,OUTPUT);
  pinMode(kD4DataInput,OUTPUT);
  pinMode(kD5DataInput,OUTPUT);
  pinMode(kD6DataInput,OUTPUT);
  pinMode(kA0DigitSelect,OUTPUT);
  pinMode(kA1DigitSelect,OUTPUT);
  pinMode(kWRWrite,OUTPUT);
  clearDigits();
}

// public methods
void DL1414T::setDigitRaw(int digit, boolean bit6, boolean bit5, boolean bit4, boolean bit3, boolean bit2, boolean bit1, boolean bit0)
{
  // select digit
  switch (digit) {
    case 0:
      digitalWrite(kA0DigitSelect,LOW);
      digitalWrite(kA1DigitSelect,LOW);
      break;
    case 1:
      digitalWrite(kA0DigitSelect,HIGH);
      digitalWrite(kA1DigitSelect,LOW);
      break;
    case 2:
      digitalWrite(kA0DigitSelect,LOW);
      digitalWrite(kA1DigitSelect,HIGH);
      break;
    case 3:
      digitalWrite(kA0DigitSelect,HIGH);
      digitalWrite(kA1DigitSelect,HIGH);
      break;
  }

  // enable write
  digitalWrite(kWRWrite,LOW);

  // set datalines
  digitalWrite(kD0DataInput,bit0);
  digitalWrite(kD1DataInput,bit1);
  digitalWrite(kD2DataInput,bit2);
  digitalWrite(kD3DataInput,bit3);
  digitalWrite(kD4DataInput,bit4);
  digitalWrite(kD5DataInput,bit5);
  digitalWrite(kD6DataInput,bit6);

  // disable write
  digitalWrite(kWRWrite,HIGH);
}

void DL1414T::setDigit(int digit, char character)
{
  // convert character code to bits
  int charCode = (int) toupper(character);
  int c = charCode;
  int bit6 = c / 64;
  c = c % 64;
  int bit5 = c / 32;
  c = c % 32;
  int bit4 = c / 16;
  c = c % 16;
  int bit3 = c / 8;
  c = c % 8;
  int bit2 = c / 4;
  c = c % 4;
  int bit1 = c / 2;
  c = c % 2;
  int bit0 = c / 1;

  setDigitRaw(digit,bit6,bit5,bit4,bit3,bit2,bit1,bit0);
}

void DL1414T::clearDigits()
{
  for (int digit = 0; digit < 4; ++digit) {
    setDigit(digit,' ');
  }
}

void DL1414T::showStr4(char c3, char c2, char c1, char c0)
{
  setDigit(3,c3);
  setDigit(2,c2);
  setDigit(1,c1);
  setDigit(0,c0);
}

void DL1414T::showStr4(String s)
{
  showStr4(s[0],s[1],s[2],s[3]);
}

void DL1414T::showInt(int i)
{
  if (i >= 10000) {
    showStr4('-','-','-','-');
  }
  else if (i >= 1000) {
    int thousands = i / 1000;
    i = i - (thousands * 1000);
    int hundreds = i / 100;
    i = i - (hundreds * 100);
    int tenths = i / 10;
    i = i - (tenths * 10);
    setDigit(3,(char) (((int) '0') + thousands));
    setDigit(2,(char) (((int) '0') + hundreds));
    setDigit(1,(char) (((int) '0') + tenths));
    setDigit(0,(char) (((int) '0') + i));
  }
  else if (i >= 100) {
    int hundreds = i / 100;
    i = i - (hundreds * 100);
    int tenths = i / 10;
    i = i - (tenths * 10);
    setDigit(3,' ');
    setDigit(2,(char) (((int) '0') + hundreds));
    setDigit(1,(char) (((int) '0') + tenths));
    setDigit(0,(char) (((int) '0') + i));
  }
  else if (i >= 10) {
    int tenths = i / 10;
    i = i - (tenths * 10);
    setDigit(3,' ');
    setDigit(2,' ');
    setDigit(1,(char) (((int) '0') + tenths));
    setDigit(0,(char) (((int) '0') + i));
  }
  else if (i >= 0) {
    setDigit(3,' ');
    setDigit(2,' ');
    setDigit(1,' ');
    setDigit(0,(char) (((int) '0') + i));
  }
}

void DL1414T::showFloat(float f)
{
  if (f >= 100.0f) {
    showInt((int) round(f));
  }
  else if (f >= 10.0f) {
    int tenths = f / 10;
    int units = f - (tenths * 10);
    setDigit(3,(char) (((int) '0') + tenths));
    setDigit(2,(char) (((int) '0') + units));
    setDigit(1,',');
    float fractionalPart = f - floor(f);
    fractionalPart = round(fractionalPart * 10);
    setDigit(0,(char) (((int) '0') + fractionalPart));
  }
  else if (f >= 0.0f) {
    int units = floor(f);
    setDigit(3,(char) (((int) '0') + units));
    setDigit(2,',');
    float fractionalPart = f - floor(f);
    fractionalPart = round(fractionalPart * 100);
    int tenths = fractionalPart / 10;
    units = fractionalPart - (tenths * 10);
    setDigit(1,(char) (((int) '0') + tenths));
    setDigit(0,(char) (((int) '0') + units));
  }
}

void DL1414T::ticker(String s)
{
  int waitTime = 200;

  int charPtr = 0;
  do {
    for (int i = 0; i < 4; ++i) {
      setDigit(3 - i,s[charPtr + i]);
    }
    if (charPtr == 0) {
      // initial delay
      delay(waitTime * 3);
    } 
    ++charPtr;
    delay(waitTime);
  }
  while (!((charPtr + 3) > s.length()));
  delay(waitTime * 3);
}
